--[[
  < ResourcesOnTop v0.3 Beta >
_________________________________
    Author: Spyro
Maintainer: Selfron
   License: All Rights Reserved
   Contact: Spyr  @ ArenaJunkies
            Spyro  @ WowInterface
            Spyro_ @ Curse/WowAce

]]

local _ -- Avoiding tainting Blizzard's global declaration of _
local Addon = CreateFrame("Frame") -- Addon's main frame
local PLAYER_CLASS = select(2, UnitClass("player"))

-- Event registration
Addon:SetScript("OnEvent", function(self, event, ...) return self[event](self, ...) end)
Addon:RegisterEvent("PLAYER_LOGIN")

-- yOffset()
-- Changes the vertical offset of the anchor of a frame.
-- > Frame: Frame reference
-- > Offset: Vertical offset
local function yOffset(Frame, Offset)
  local P = { Frame:GetPoint() }
  Frame:SetPoint(P[1], P[2], P[3], P[4], Offset)
end

-- yInvertTexture()
-- Inverts a texture vertically.
-- > Texture: Texture reference
local function yInvertTexture(Texture)
  local Left, Top, _, Bottom, Right = Texture:GetTexCoord()
  Texture:SetTexCoord(Left, Right, Bottom, Top) -- Swapping parameters 3 & 4 (top & bottom)
end

-- yInvertAllTextures()
-- Inverts vertically all the child textures of a frame.
-- > Frame: Frame reference
local function yInvertAllTextures(Frame)
  for _, Region in pairs({ Frame:GetRegions() }) do
    if Region:IsObjectType("Texture") then yInvertTexture(Region) end
  end
end

-- Event LEARNED_SPELL_IN_TAB
-- Fires when a spell is learned inside of spell book tab, useful to detect talent spec changes.
function Addon:LEARNED_SPELL_IN_TAB(SpellID, TabID)
  -- Enhanced Shadow Orbs perk, inverting the 5 small orbs
  if SpellID == SHADOW_ORB_MINOR_TALENT_ID then
    yOffset(PriestBarFrame.SmallOrbs[1], -22)
    for i = 1, 5 do yInvertAllTextures(PriestBarFrame.SmallOrbs[i]) end
    Addon:UnregisterEvent("LEARNED_SPELL_IN_TAB") -- We only need to invert it the first time the frame is created
  end
end

-- Event PLAYER_LOGIN
-- Fires after PLAYER_ENTERING_WORLD after logging in and after /reloadui.
function Addon:PLAYER_LOGIN()
  -- Paladin Holy Power
  if PLAYER_CLASS == "PALADIN" then
    yOffset(PaladinPowerBar, 111)
    yOffset(PaladinPowerBarRune1, -5)
    yOffset(PaladinPowerBarRune4, 3)
    yOffset(PaladinPowerBarRune5, 3)
    yOffset(PaladinPowerBarBankBG, 7)
    yInvertAllTextures(PaladinPowerBar)
    yInvertTexture(PaladinPowerBarGlowBGTexture)
    for i = 1, 5 do yInvertTexture(_G["PaladinPowerBarRune"..i.."Texture"]) end

  -- Shadow Priest Orbs
  elseif PLAYER_CLASS == "PRIEST" then
    Addon:RegisterEvent("LEARNED_SPELL_IN_TAB") -- To detect Enhanced Shadow Orbs perk
    yInvertAllTextures(PriestBarFrame)
    yOffset(PriestBarFrame, 131)

    if IsSpellKnown(SHADOW_ORB_MINOR_TALENT_ID) then -- If the Enhanced Shadow Orbs perk is enabled, invert the 5 small orbs
      Addon:LEARNED_SPELL_IN_TAB(SHADOW_ORB_MINOR_TALENT_ID)
    else -- No perk, invert the 3 large orbs
      yOffset(PriestBarFrame.LargeOrbs[1], -16)
      for i = 1, 3 do
        yInvertAllTextures(PriestBarFrame.LargeOrbs[i])
        yOffset(PriestBarFrame.LargeOrbs[i].Highlight, 8)
      end
    end

  -- Warlock
  elseif PLAYER_CLASS == "WARLOCK" then
    -- Affliction Shards
    yOffset(ShardBarFrame, 69)
    for i = 1, 4 do
      yInvertAllTextures(_G["ShardBarFrameShard"..i])
      yOffset(_G["ShardBarFrameShard"..i].shardFill, -5)
    end

    -- Destruction Burning Embers
    yOffset(BurningEmbersBarFrame, 63)
    yOffset(BurningEmbersBarFrameEmber1, 14)
    yInvertTexture(BurningEmbersBarFrame.background)
    for i = 1, 4 do yInvertAllTextures(_G["BurningEmbersBarFrameEmber"..i]) end

    -- Demonology Demonic Fury
    yOffset(DemonicFuryBarFrame, 82)

  -- DK Runes
  elseif PLAYER_CLASS == "DEATHKNIGHT" then
    yOffset(RuneFrame, 99)

  -- Druid Eclipse
  elseif PLAYER_CLASS == "DRUID" then
    yOffset(EclipseBarFrame, 108)

    -- Druid AlternatePowerBar
    yInvertAllTextures(PlayerFrameAlternateManaBar)
    yOffset(PlayerFrameAlternateManaBar, 79)
    yOffset(PlayerFrameAlternateManaBar.DefaultBorder, 4)
    yOffset(PlayerFrameAlternateManaBar.MonkBorder, 4)
    yOffset(PlayerFrameAlternateManaBar.MonkBackground, 4)
    local Offset = 79
    if MonkHarmonyBar:IsShown() then Offset = 100 end
    hooksecurefunc(PlayerFrameAlternateManaBar, "SetPoint", function(Bar) yOffset(Bar, Offset) end)

  -- Monk Chi
  elseif PLAYER_CLASS == "MONK" then
    yOffset(MonkHarmonyBar, 20)
    yInvertAllTextures(MonkHarmonyBar)

    -- Monk Stagger Bar
    yOffset(MonkStaggerBarText, -4)
    yInvertAllTextures(MonkStaggerBar)
    hooksecurefunc(MonkStaggerBar, "SetPoint", function(Bar) yOffset(Bar, 104) end)
  end
  
  -- Shaman Totems / Mage Rune of Power / Druid Mushrooms
  if PLAYER_CLASS == "SHAMAN" or PLAYER_CLASS == "MAGE" or PLAYER_CLASS == "DRUID" then
    TotemFrame:SetPoint("TOPLEFT", PlayerFrame, "BOTTOMLEFT", 99, 112)
    TotemFrame.SetPoint = function() end
    for i = 1, 4 do
      yOffset(_G["TotemFrameTotem"..i.."Duration"], 44)
      yInvertTexture(select(2, _G["TotemFrameTotem"..i]:GetChildren()):GetRegions())
    end
  end
end